const { VM } = require('vm2');
const logger = require('../../utils/logger');

/**
 * Function Node Runner
 * Executes custom JavaScript code in a sandboxed environment
 */
async function run(nodeConfig, inputData) {
  try {
    logger.debug('Function node executing');
    
    // Check required configuration
    if (!nodeConfig.code) {
      throw new Error('Missing function code');
    }
    
    // Set timeout (default: 5000ms)
    const timeout = nodeConfig.timeout || 5000;
    
    // Create a sandboxed VM
    const vm = new VM({
      timeout,
      sandbox: {
        // Provide input data to the sandbox
        input: inputData.input || {},
        // Provide console methods that log to our logger
        console: {
          log: (...args) => logger.info('Function node log:', ...args),
          info: (...args) => logger.info('Function node info:', ...args),
          warn: (...args) => logger.warn('Function node warn:', ...args),
          error: (...args) => logger.error('Function node error:', ...args)
        }
      },
      // Prevent access to Node.js internal modules
      require: {
        external: false,
        builtin: ['path', 'util', 'buffer'],
        root: "./",
        mock: {
          fs: {
            readFileSync: () => 'Not allowed'
          }
        }
      }
    });
    
    // Wrap the code in an async function
    const wrappedCode = `
      (async function() {
        ${nodeConfig.code}
      })();
    `;
    
    // Execute the code
    const result = await vm.run(wrappedCode);
    
    // Validate the result
    if (result === undefined || result === null) {
      return {
        output: {}
      };
    }
    
    // Return the result
    return {
      output: result
    };
  } catch (error) {
    logger.error('Function node error', { error: error.message });
    
    // Return error through the error output
    return {
      error: {
        message: error.message,
        stack: error.stack
      }
    };
  }
}

module.exports = { run };
